@@ -1,7 +1,7 @@
 /*
  *  query.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -83,8 +83,10 @@ static int search_path(char **filename, struct stat *bufptr)
 static void print_query_fileowner(const char *filename, alpm_pkg_t *info)
 {
 	if(!config->quiet) {
-		printf(_("%s is owned by %s %s\n"), filename,
-				alpm_pkg_get_name(info), alpm_pkg_get_version(info));
+		const colstr_t *colstr = &config->colstr;
+		printf(_("%s is owned by %s%s %s%s%s\n"), filename, colstr->title,
+				alpm_pkg_get_name(info), colstr->version, alpm_pkg_get_version(info),
+				colstr->nocolor);
 	} else {
 		printf("%s\n", alpm_pkg_get_name(info));
 	}
@@ -156,38 +158,38 @@ static int query_fileowner(alpm_list_t *targets)
 		char rpath[PATH_MAX], *rel_path;
 		struct stat buf;
 		alpm_list_t *i;
-		size_t len, is_dir;
+		size_t len;
 		unsigned int found = 0;
+		int is_dir = 0, is_missing = 0;
 
 		if((filename = strdup(t->data)) == NULL) {
 			goto targcleanup;
 		}
 
+		if(strcmp(filename, "") == 0) {
+			pm_printf(ALPM_LOG_ERROR, _("empty string passed to file owner query\n"));
+			goto targcleanup;
+		}
+
 		/* trailing '/' causes lstat to dereference directory symlinks */
 		len = strlen(filename) - 1;
 		while(len > 0 && filename[len] == '/') {
 			filename[len--] = '\0';
+			/* If a non-dir file exists, S_ISDIR will correct this later. */
+			is_dir = 1;
 		}
 
 		if(lstat(filename, &buf) == -1) {
+			is_missing = 1;
 			/* if it is not a path but a program name, then check in PATH */
-			if(strchr(filename, '/') == NULL) {
-				if(search_path(&filename, &buf) == -1) {
-					pm_printf(ALPM_LOG_ERROR, _("failed to find '%s' in PATH: %s\n"),
-							filename, strerror(errno));
-					goto targcleanup;
-				}
-			} else {
-				pm_printf(ALPM_LOG_ERROR, _("failed to read file '%s': %s\n"),
-						filename, strerror(errno));
-				goto targcleanup;
+			if ((strchr(filename, '/') == NULL) && (search_path(&filename, &buf) == 0)) {
+				is_missing = 0;
 			}
 		}
 
 		if(!lrealpath(filename, rpath)) {
-			pm_printf(ALPM_LOG_ERROR, _("cannot determine real path for '%s': %s\n"),
-					filename, strerror(errno));
-			goto targcleanup;
+			/* Can't canonicalize path, try to proceed anyway */
+			strncpy(rpath, filename, PATH_MAX);
 		}
 
 		if(strncmp(rpath, root, rootlen) != 0) {
@@ -198,10 +200,11 @@ static int query_fileowner(alpm_list_t *targets)
 
 		rel_path = rpath + rootlen;
 
-		if((is_dir = S_ISDIR(buf.st_mode))) {
+		if((is_missing && is_dir) || (!is_missing && (is_dir = S_ISDIR(buf.st_mode)))) {
 			size_t rlen = strlen(rpath);
 			if(rlen + 2 >= PATH_MAX) {
 					pm_printf(ALPM_LOG_ERROR, _("path too long: %s/\n"), rpath);
+					goto targcleanup;
 			}
 			strcat(rpath + rlen, "/");
 		}
@@ -460,27 +463,30 @@ int pacman_query(alpm_list_t *targets)
 
 		if(config->op_q_isfile) {
 			alpm_pkg_load(config->handle, strname, 1, 0, &pkg);
+
+			if(pkg == NULL) {
+				pm_printf(ALPM_LOG_ERROR,
+						_("could not load package '%s': %s\n"), strname,
+						alpm_strerror(alpm_errno(config->handle)));
+			}
 		} else {
 			pkg = alpm_db_get_pkg(db_local, strname);
+			if(pkg == NULL) {
+				pkg = alpm_find_satisfier(alpm_db_get_pkgcache(db_local), strname);
+			}
+
+			if(pkg == NULL) {
+				pm_printf(ALPM_LOG_ERROR,
+						_("package '%s' was not found\n"), strname);
+				if(!config->op_q_isfile && access(strname, R_OK) == 0) {
+					pm_printf(ALPM_LOG_WARNING,
+							_("'%s' is a file, you might want to use %s.\n"),
+							strname, "-p/--file");
+				}
+			}
 		}
 
 		if(pkg == NULL) {
-			switch(alpm_errno(config->handle)) {
-				case ALPM_ERR_PKG_NOT_FOUND:
-					pm_printf(ALPM_LOG_ERROR,
-							_("package '%s' was not found\n"), strname);
-					if(!config->op_q_isfile && access(strname, R_OK) == 0) {
-						pm_printf(ALPM_LOG_WARNING,
-								_("'%s' is a file, you might want to use %s.\n"),
-								strname, "-p/--file");
-					}
-					break;
-				default:
-					pm_printf(ALPM_LOG_ERROR,
-							_("could not load package '%s': %s\n"), strname,
-							alpm_strerror(alpm_errno(config->handle)));
-					break;
-			}
 			ret = 1;
 			continue;
 		}
@@ -505,5 +511,3 @@ int pacman_query(alpm_list_t *targets)
 
 	return ret;
 }
-
-/* vim: set noet: */
