@@ -2,7 +2,7 @@
  *  vercmp.c - Compare package version numbers using pacman's version
  *      comparison logic
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2005 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -40,8 +40,6 @@ static void usage(void)
 
 int main(int argc, char *argv[])
 {
-	const char *s1 = "";
-	const char *s2 = "";
 	int ret;
 
 	if(argc == 1) {
@@ -49,19 +47,17 @@ int main(int argc, char *argv[])
 		return 2;
 	}
 	if(argc > 1 &&
-			(strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0
-			 || strcmp(argv[1], "--usage") == 0)) {
+			(strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0)) {
 		usage();
 		return 0;
 	}
-	if(argc > 2) {
-		s2 = argv[2];
-	}
-	if(argc > 1) {
-		s1 = argv[1];
+	if(argc != 3) {
+		fprintf(stderr, "error: %d argument(s) specified\n\n"
+			"Usage: vercmp <ver1> <ver2>\n", argc - 1);
+		return EXIT_FAILURE;
 	}
 
-	ret = alpm_pkg_vercmp(s1, s2);
+	ret = alpm_pkg_vercmp(argv[1], argv[2]);
 	printf("%d\n", ret);
 	return EXIT_SUCCESS;
 }
