@@ -11,7 +11,3 @@ self.args = "-R %s" % p1.name
 self.addrule("PACMAN_RETCODE=0")
 self.addrule("FILE_EXIST=pre_remove")
 self.addrule("FILE_EXIST=post_remove")
-
-fakechroot = util.which("fakechroot")
-if not fakechroot:
-	self.expectfailure = True
